/*
 * Daneel - Dalvik to Java bytecode compiler
 * Copyright (C) 2011  IcedRobot team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * This file is subject to the "Classpath" exception:
 *
 * Linking this library statically or dynamically with other modules is
 * making a combined work based on this library.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under terms
 * of your choice, provided that you also meet, for each linked independent
 * module, the terms and conditions of the license of that module.  An
 * independent module is a module which is not derived from or based on
 * this library.  If you modify this library, you may extend this exception
 * to your version of the library, but you are not obligated to do so.  If
 * you do not wish to do so, delete this exception statement from your
 * version.
 */

package org.icedrobot.daneel.rewriter;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertEquals;

import org.icedrobot.daneel.DexifyingRunner;
import org.junit.Test;
import org.junit.runner.RunWith;

@RunWith(DexifyingRunner.class)
public class ArrayAccessTest {

    @Test
    public void testInt() {
        int[] array = new int[3];
        DEXCode.aput(array, 0, 23);
        DEXCode.aput(array, 1, 42);
        assertArrayEquals(new int[] { 23, 42, 0 }, array);
        assertEquals(23, DEXCode.aget(array, 0));
        assertEquals(42, DEXCode.aget(array, 1));
        assertEquals(0, DEXCode.aget(array, 2));
    }

    @Test
    public void testObject() {
        Object obj1 = new String("some test object");
        Object obj2 = new String("another test object");
        Object[] array = new Object[3];
        DEXCode.aput(array, 0, obj1);
        DEXCode.aput(array, 1, obj2);
        assertArrayEquals(new Object[] { obj1, obj2, null }, array);
        assertEquals(obj1, DEXCode.aget(array, 0));
        assertEquals(obj2, DEXCode.aget(array, 1));
        assertEquals(null, DEXCode.aget(array, 2));
    }

    // Keep this class named "DEXCode" to push it through Daneel.
    private static class DEXCode {

        public static int aget(int[] a, int i) { return a[i]; }
        public static void aput(int[] a, int i, int v) { a[i] = v; }

        public static Object aget(Object[] a, int i) { return a[i]; }
        public static void aput(Object[] a, int i, Object v) { a[i] = v; }

    };
}
